@@ -27,6 +27,8 @@
 #include <qbitarray.h>
 #include <qptrlist.h>
 
+#include "libkcal_export.h"
+
 namespace KCal {
 
 class Incidence;
@@ -34,7 +36,7 @@ class Incidence;
 /**
   This class represents a recurrence rule for a calendar incidence.
 */
-class Recurrence
+class LIBKCAL_EXPORT Recurrence
 {
   public:
     /** enumeration for describing how an event recurs, if at all. */
@@ -80,7 +82,7 @@ class Recurrence
      * N.B. This property is derived by default from the parent incidence,
      * or according to whether a time is specified in setRecurStart(). */
     void setFloats(bool f);
-    /** 
+    /**
       Returns whether the recurrence has no time, just a date.
     */
     bool doesFloat() const { return mFloats; }
@@ -148,7 +150,13 @@ class Recurrence
 
     /** Returns frequency of recurrence, in terms of the recurrence time period type. */
     int frequency() const;
-    /** Returns the total number of recurrences, including the initial occurrence. */
+    /** Sets the frequency of recurrence, in terms of the recurrence time period type. */
+    void setFrequency(int freq);
+
+    /**
+     * Returns -1 if the event recurs infinitely, 0 if the end date is set,
+     * otherwise the total number of recurrences, including the initial occurrence.
+     */
     int duration() const;
     /** Sets the total number of times the event is to occur, including both the
      * first and last. */
@@ -173,6 +181,12 @@ class Recurrence
     /** Returns a string representing the recurrence end date in the format
      according to the user's locale settings. */
     QString endDateStr(bool shortfmt=true) const;
+    /** Sets the date of the last recurrence. The end time is set to the recurrence start time.
+     * @param endDate the ending date after which to stop recurring. */
+    void setEndDate(const QDate &endDate);
+    /** Sets the date and time of the last recurrence.
+     * @param endDateTime the ending date/time after which to stop recurring. */
+    void setEndDateTime(const QDateTime &endDateTime);
 
     /** Sets an event to recur minutely.
      * @param _rFreq the frequency to recur, e.g. 2 is every other minute
@@ -243,7 +257,7 @@ class Recurrence
      * the rule is to repeat every 2nd Monday and Wednesday in the month.
      */
     void addMonthlyPos(short _rPos, const QBitArray &_rDays);
-    /** Adds a position the the recursMonthlyDay list.
+    /** Adds a position to the recursMonthlyDay list.
      * @param _rDay the date in the month to recur.
      */
     void addMonthlyDay(short _rDay);
@@ -300,6 +314,37 @@ class Recurrence
     /** Returns the default method for handling yearly recurrences of February 29th. */
     static Feb29Type setFeb29YearlyTypeDefault()  { return mFeb29YearlyDefaultType; }
 
+    /** Returns the number of days in the month which recur.
+     * Reply = -1 if the number varies from month to month.
+     */
+    int  countMonthlyPosDays() const;
+    /** Returns the days in a specified month which recur, in numerical order.
+     * @param list receives the list of days which recur
+     * @param daysInMonth number of days in this month
+     * @param startDayOfWeek day of week for the first day in this month.
+     */
+    void getMonthlyPosDays(QValueList<int>& list, int daysInMonth,
+                           int startDayOfWeek) const;
+    /** Returns the days in a specified month which recur, in numerical order.
+     * @param list receives the list of days which recur
+     * @param daysInMonth number of days in this month
+     * Reply = true if day numbers varies from month to month.
+     */
+    bool getMonthlyDayDays(QValueList<int>& list, int daysInMonth) const;
+    /** Get the months which recur, in numerical order, for both leap years and non-leap years.
+     * N.B. If February 29th recurs on March 1st in non-leap years, February (not March) is
+     * included in the non-leap year month list.
+     * @param day
+     * @param list receives the list of months which recur in non-leap years
+     * @param leaplist receives the list of months which recur in leap years
+     * Reply = true if February 29th also recurs.
+     */
+    bool getYearlyMonthMonths(int day, QValueList<int>& list,
+                              QValueList<int> &leaplist) const;
+
+    /** Upper date limit for recurrences */
+    static const QDate MAX_DATE;
+
     /**
       Debug output.
     */
@@ -353,13 +398,6 @@ class Recurrence
     int  yearlyDayCalcToDate(const QDate& enddate, YearlyDayData&) const;
     int  yearlyDayCalcNextAfter(QDate& enddate, YearlyDayData&) const;
 
-  public:
-    int  countMonthlyPosDays() const;
-    void getMonthlyPosDays(QValueList<int>&, int daysInMonth,
-                           int startDayOfWeek) const;
-    bool getMonthlyDayDays(QValueList<int>&, int daysInMonth) const;
-    bool getYearlyMonthMonths(int day, QValueList<int>&,
-                              QValueList<int> &leaplist) const;
   private:
 
     int   getFirstDayInWeek(int startDay, bool useWeekStart = true) const;
@@ -368,8 +406,8 @@ class Recurrence
     QDate getLastDateInMonth(const QDate& latestDate) const;
     QDate getFirstDateInYear(const QDate& earliestDate) const;
     QDate getLastDateInYear(const QDate& latestDate) const;
+    int   maxIterations() const;
 
-  private:
     // Prohibit copying
     Recurrence(const Recurrence&);
     Recurrence &operator=(const Recurrence&);
@@ -391,7 +429,7 @@ class Recurrence
     int rFreq;                           // frequency of period
 
     // one of the following must be specified
-    int rDuration;                       // num times to recur (inc. first occurrence), -1 = infinite
+    int rDuration;                       // num times to recur (inc. first occurrence), -1 = infinite, 0 means end date is set
     QDateTime rEndDateTime;              // date/time at which to end recurrence
     mutable QDateTime mCachedEndDT;      // calculated end date/time for duration > 0
                                          // (calculation can be significant for some recurrences)
